'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const LinkedList = require("linked-list");
const linked_list_1 = require("linked-list");
class Node extends linked_list_1.Item {
    constructor(item) {
        super();
        this.item = item;
    }
}
class BlockingQueue extends events_1.EventEmitter {
    constructor(options) {
        super();
        this._queue = new LinkedList();
        this._activeCount = 0;
        options = options || {};
        this._options = Object.assign({}, options);
        if (!(typeof this._options.concurrency === 'number' && this._options.concurrency >= 1)) {
            throw new TypeError(`Expected \`concurrency\` to be a number from 1 and up, got \`${this._options.concurrency}\` (${typeof this._options.concurrency})`);
        }
        this._boundNext = this._next.bind(this);
    }
    enqueue(fn, ...args) {
        const fnPromiseParts = this._getPromiseParts();
        const enqueuePromiseParts = this._getPromiseParts();
        const item = {
            fn,
            args,
            fnResolve: fnPromiseParts.resolve,
            enqueueResolve: enqueuePromiseParts.resolve,
        };
        if (this.activeCount < this._options.concurrency) {
            this._run(item);
        }
        else {
            this._queue.append(new Node(item));
        }
        return {
            enqueuePromise: enqueuePromiseParts.promise,
            fnPromise: fnPromiseParts.promise,
        };
    }
    get activeCount() {
        return this._activeCount;
    }
    get pendingCount() {
        return this._queue.size;
    }
    _next() {
        this._activeCount--;
        const node = this._queue.head;
        if (node) {
            node.detach();
            this._run(node.item);
        }
        else {
            this.emit('empty');
            if (this._activeCount === 0) {
                this.emit('idle');
            }
        }
    }
    _run(item) {
        this._activeCount++;
        item.enqueueResolve();
        const result = Promise.resolve()
            .then(() => {
            return Promise.resolve(item.fn(...item.args));
        });
        item.fnResolve(result);
        result.then(this._boundNext, this._boundNext);
    }
    _getPromiseParts() {
        let resolve;
        const promise = new Promise((_resolve) => {
            resolve = _resolve;
        });
        return {
            promise,
            // @ts-ignore
            resolve,
        };
    }
}
exports.BlockingQueue = BlockingQueue;
//# sourceMappingURL=BlockingQueue.js.map